# -*- coding: utf-8 -*-
from .._base import AsyncBase
from ..generated import calibration_pb2, calibration_pb2_grpc
from enum import Enum


class CalibrationResult:
    """
     Result type.

     Parameters
     ----------
     result : Result
          Result enum value

     result_str : std::string
          Human-readable English string describing the result

     """

    
    
    class Result(Enum):
        """
         Possible results returned for calibration commands

         Values
         ------
         UNKNOWN
              Unknown error

         SUCCESS
              The calibration process succeeded

         IN_PROGRESS
              Intermediate message showing progress of the calibration process

         INSTRUCTION
              Intermediate message giving instructions on the next steps required by the process

         FAILED
              Calibration failed

         NO_SYSTEM
              No system is connected

         CONNECTION_ERROR
              Connection error

         BUSY
              Vehicle is busy

         COMMAND_DENIED
              Command refused by vehicle

         TIMEOUT
              Command timed out

         CANCELLED
              Calibration process got cancelled

         """

        
        UNKNOWN = 0
        SUCCESS = 1
        IN_PROGRESS = 2
        INSTRUCTION = 3
        FAILED = 4
        NO_SYSTEM = 5
        CONNECTION_ERROR = 6
        BUSY = 7
        COMMAND_DENIED = 8
        TIMEOUT = 9
        CANCELLED = 10

        def translate_to_rpc(self, rpcResult):
            return {
                    0: calibration_pb2.CalibrationResult.UNKNOWN,
                    1: calibration_pb2.CalibrationResult.SUCCESS,
                    2: calibration_pb2.CalibrationResult.IN_PROGRESS,
                    3: calibration_pb2.CalibrationResult.INSTRUCTION,
                    4: calibration_pb2.CalibrationResult.FAILED,
                    5: calibration_pb2.CalibrationResult.NO_SYSTEM,
                    6: calibration_pb2.CalibrationResult.CONNECTION_ERROR,
                    7: calibration_pb2.CalibrationResult.BUSY,
                    8: calibration_pb2.CalibrationResult.COMMAND_DENIED,
                    9: calibration_pb2.CalibrationResult.TIMEOUT,
                    10: calibration_pb2.CalibrationResult.CANCELLED
                }.get(self.value, None)

        @staticmethod
        def translate_from_rpc(rpc_enum_value):
            """ Parses a gRPC response """
            return {
                    0: CalibrationResult.Result.UNKNOWN,
                    1: CalibrationResult.Result.SUCCESS,
                    2: CalibrationResult.Result.IN_PROGRESS,
                    3: CalibrationResult.Result.INSTRUCTION,
                    4: CalibrationResult.Result.FAILED,
                    5: CalibrationResult.Result.NO_SYSTEM,
                    6: CalibrationResult.Result.CONNECTION_ERROR,
                    7: CalibrationResult.Result.BUSY,
                    8: CalibrationResult.Result.COMMAND_DENIED,
                    9: CalibrationResult.Result.TIMEOUT,
                    10: CalibrationResult.Result.CANCELLED,
                }.get(rpc_enum_value, None)

        def __str__(self):
            return self.name
    

    def __init__(
            self,
            result,
            result_str):
        """ Initializes the CalibrationResult object """
        self.result = result
        self.result_str = result_str

    def __equals__(self, to_compare):
        """ Checks if two CalibrationResult are the same """
        try:
            # Try to compare - this likely fails when it is compared to a non
            # CalibrationResult object
            return \
                (self.result == to_compare.result) and \
                (self.result_str == to_compare.result_str)

        except AttributeError:
            return False

    def __str__(self):
        """ CalibrationResult in string representation """
        struct_repr = ", ".join([
                "result: " + str(self.result),
                "result_str: " + str(self.result_str)
                ])

        return f"CalibrationResult: [{struct_repr}]"

    @staticmethod
    def translate_from_rpc(rpcCalibrationResult):
        """ Translates a gRPC struct to the SDK equivalent """
        return CalibrationResult(
                
                CalibrationResult.Result.translate_from_rpc(rpcCalibrationResult.result),
                
                
                rpcCalibrationResult.result_str
                )

    def translate_to_rpc(self, rpcCalibrationResult):
        """ Translates this SDK object into its gRPC equivalent """

        
        
            
        self.result.translate_to_rpc(rpcCalibrationResult.result)
            
        
        
        
            
        rpcCalibrationResult.result_str = self.result_str
            
        
        


class ProgressData:
    """
     Progress data coming from calibration.

     Can be a progress percentage, or an instruction text.

     Parameters
     ----------
     has_progress : bool
         
     progress : float
          Progress (percentage)

     has_status_text : bool
         
     status_text : std::string
          Instruction text

     """

    

    def __init__(
            self,
            has_progress,
            progress,
            has_status_text,
            status_text):
        """ Initializes the ProgressData object """
        self.has_progress = has_progress
        self.progress = progress
        self.has_status_text = has_status_text
        self.status_text = status_text

    def __equals__(self, to_compare):
        """ Checks if two ProgressData are the same """
        try:
            # Try to compare - this likely fails when it is compared to a non
            # ProgressData object
            return \
                (self.has_progress == to_compare.has_progress) and \
                (self.progress == to_compare.progress) and \
                (self.has_status_text == to_compare.has_status_text) and \
                (self.status_text == to_compare.status_text)

        except AttributeError:
            return False

    def __str__(self):
        """ ProgressData in string representation """
        struct_repr = ", ".join([
                "has_progress: " + str(self.has_progress),
                "progress: " + str(self.progress),
                "has_status_text: " + str(self.has_status_text),
                "status_text: " + str(self.status_text)
                ])

        return f"ProgressData: [{struct_repr}]"

    @staticmethod
    def translate_from_rpc(rpcProgressData):
        """ Translates a gRPC struct to the SDK equivalent """
        return ProgressData(
                
                rpcProgressData.has_progress,
                
                
                rpcProgressData.progress,
                
                
                rpcProgressData.has_status_text,
                
                
                rpcProgressData.status_text
                )

    def translate_to_rpc(self, rpcProgressData):
        """ Translates this SDK object into its gRPC equivalent """

        
        
            
        rpcProgressData.has_progress = self.has_progress
            
        
        
        
            
        rpcProgressData.progress = self.progress
            
        
        
        
            
        rpcProgressData.has_status_text = self.has_status_text
            
        
        
        
            
        rpcProgressData.status_text = self.status_text
            
        
        



class CalibrationError(Exception):
    """ Raised when a CalibrationResult is a fail code """

    def __init__(self, result, origin, *params):
        self._result = result
        self._origin = origin
        self._params = params

    def __str__(self):
        return f"{self._result.result}: '{self._result.result_str}'; origin: {self._origin}; params: {self._params}"


class Calibration(AsyncBase):
    """
     Enable to calibrate sensors of a drone such as gyro, accelerometer, and magnetometer.

     Generated by dcsdkgen - MAVSDK Calibration API
    """

    # Plugin name
    name = "Calibration"

    def _setup_stub(self, channel):
        """ Setups the api stub """
        self._stub = calibration_pb2_grpc.CalibrationServiceStub(channel)

    
    def _extract_result(self, response):
        """ Returns the response status and description """
        return CalibrationResult.translate_from_rpc(response.calibration_result)
    

    async def calibrate_gyro(self):
        """
         Perform gyro calibration.

         Yields
         -------
         progress_data : ProgressData
              Progress data

         Raises
         ------
         CalibrationError
             If the request fails. The error contains the reason for the failure.
        """

        request = calibration_pb2.SubscribeCalibrateGyroRequest()
        calibrate_gyro_stream = self._stub.SubscribeCalibrateGyro(request)

        try:
            async for response in calibrate_gyro_stream:
                
                result = self._extract_result(response)

                success_codes = [CalibrationResult.Result.SUCCESS]
                if 'IN_PROGRESS' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.IN_PROGRESS)
                if 'INSTRUCTION' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.INSTRUCTION)

                if result.result not in success_codes:
                    raise CalibrationError(result, "calibrate_gyro()")

                if result.result is CalibrationResult.Result.SUCCESS:
                    calibrate_gyro_stream.cancel();
                    return
                

            
                yield ProgressData.translate_from_rpc(response.progress_data)
        finally:
            calibrate_gyro_stream.cancel()

    async def calibrate_accelerometer(self):
        """
         Perform accelerometer calibration.

         Yields
         -------
         progress_data : ProgressData
              Progress data

         Raises
         ------
         CalibrationError
             If the request fails. The error contains the reason for the failure.
        """

        request = calibration_pb2.SubscribeCalibrateAccelerometerRequest()
        calibrate_accelerometer_stream = self._stub.SubscribeCalibrateAccelerometer(request)

        try:
            async for response in calibrate_accelerometer_stream:
                
                result = self._extract_result(response)

                success_codes = [CalibrationResult.Result.SUCCESS]
                if 'IN_PROGRESS' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.IN_PROGRESS)
                if 'INSTRUCTION' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.INSTRUCTION)

                if result.result not in success_codes:
                    raise CalibrationError(result, "calibrate_accelerometer()")

                if result.result is CalibrationResult.Result.SUCCESS:
                    calibrate_accelerometer_stream.cancel();
                    return
                

            
                yield ProgressData.translate_from_rpc(response.progress_data)
        finally:
            calibrate_accelerometer_stream.cancel()

    async def calibrate_magnetometer(self):
        """
         Perform magnetometer caliration.

         Yields
         -------
         progress_data : ProgressData
              Progress data

         Raises
         ------
         CalibrationError
             If the request fails. The error contains the reason for the failure.
        """

        request = calibration_pb2.SubscribeCalibrateMagnetometerRequest()
        calibrate_magnetometer_stream = self._stub.SubscribeCalibrateMagnetometer(request)

        try:
            async for response in calibrate_magnetometer_stream:
                
                result = self._extract_result(response)

                success_codes = [CalibrationResult.Result.SUCCESS]
                if 'IN_PROGRESS' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.IN_PROGRESS)
                if 'INSTRUCTION' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.INSTRUCTION)

                if result.result not in success_codes:
                    raise CalibrationError(result, "calibrate_magnetometer()")

                if result.result is CalibrationResult.Result.SUCCESS:
                    calibrate_magnetometer_stream.cancel();
                    return
                

            
                yield ProgressData.translate_from_rpc(response.progress_data)
        finally:
            calibrate_magnetometer_stream.cancel()

    async def calibrate_gimbal_accelerometer(self):
        """
         Perform gimbal accelerometer calibration.

         Yields
         -------
         progress_data : ProgressData
              Progress data

         Raises
         ------
         CalibrationError
             If the request fails. The error contains the reason for the failure.
        """

        request = calibration_pb2.SubscribeCalibrateGimbalAccelerometerRequest()
        calibrate_gimbal_accelerometer_stream = self._stub.SubscribeCalibrateGimbalAccelerometer(request)

        try:
            async for response in calibrate_gimbal_accelerometer_stream:
                
                result = self._extract_result(response)

                success_codes = [CalibrationResult.Result.SUCCESS]
                if 'IN_PROGRESS' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.IN_PROGRESS)
                if 'INSTRUCTION' in [return_code.name for return_code in CalibrationResult.Result]:
                    success_codes.append(CalibrationResult.Result.INSTRUCTION)

                if result.result not in success_codes:
                    raise CalibrationError(result, "calibrate_gimbal_accelerometer()")

                if result.result is CalibrationResult.Result.SUCCESS:
                    calibrate_gimbal_accelerometer_stream.cancel();
                    return
                

            
                yield ProgressData.translate_from_rpc(response.progress_data)
        finally:
            calibrate_gimbal_accelerometer_stream.cancel()

    async def cancel(self):
        """
         Cancel ongoing calibration process.

         
        """

        request = calibration_pb2.CancelRequest()
        response = await self._stub.Cancel(request)

        