# -*- coding: utf-8 -*-
from .._base import AsyncBase
from ..generated import gimbal_pb2, gimbal_pb2_grpc
from enum import Enum


class GimbalMode(Enum):
    """
     Gimbal mode type.

     Values
     ------
     YAW_FOLLOW
          Yaw follow will point the gimbal to the vehicle heading

     YAW_LOCK
          Yaw lock will fix the gimbal poiting to an absolute direction

     """

    
    YAW_FOLLOW = 0
    YAW_LOCK = 1

    def translate_to_rpc(self, rpcGimbalMode):
        return {
                0: gimbal_pb2.YAW_FOLLOW,
                1: gimbal_pb2.YAW_LOCK
            }.get(self.value, None)

    @staticmethod
    def translate_from_rpc(rpc_enum_value):
        """ Parses a gRPC response """
        return {
                0: GimbalMode.YAW_FOLLOW,
                1: GimbalMode.YAW_LOCK,
            }.get(rpc_enum_value, None)

    def __str__(self):
        return self.name


class GimbalResult:
    """
     Result type.

     Parameters
     ----------
     result : Result
          Result enum value

     result_str : std::string
          Human-readable English string describing the result

     """

    
    
    class Result(Enum):
        """
         Possible results returned for gimbal commands.

         Values
         ------
         UNKNOWN
              Unknown error

         SUCCESS
              Command was accepted

         ERROR
              Error occurred sending the command

         TIMEOUT
              Command timed out

         """

        
        UNKNOWN = 0
        SUCCESS = 1
        ERROR = 2
        TIMEOUT = 3

        def translate_to_rpc(self, rpcResult):
            return {
                    0: gimbal_pb2.GimbalResult.UNKNOWN,
                    1: gimbal_pb2.GimbalResult.SUCCESS,
                    2: gimbal_pb2.GimbalResult.ERROR,
                    3: gimbal_pb2.GimbalResult.TIMEOUT
                }.get(self.value, None)

        @staticmethod
        def translate_from_rpc(rpc_enum_value):
            """ Parses a gRPC response """
            return {
                    0: GimbalResult.Result.UNKNOWN,
                    1: GimbalResult.Result.SUCCESS,
                    2: GimbalResult.Result.ERROR,
                    3: GimbalResult.Result.TIMEOUT,
                }.get(rpc_enum_value, None)

        def __str__(self):
            return self.name
    

    def __init__(
            self,
            result,
            result_str):
        """ Initializes the GimbalResult object """
        self.result = result
        self.result_str = result_str

    def __equals__(self, to_compare):
        """ Checks if two GimbalResult are the same """
        try:
            # Try to compare - this likely fails when it is compared to a non
            # GimbalResult object
            return \
                (self.result == to_compare.result) and \
                (self.result_str == to_compare.result_str)

        except AttributeError:
            return False

    def __str__(self):
        """ GimbalResult in string representation """
        struct_repr = ", ".join([
                "result: " + str(self.result),
                "result_str: " + str(self.result_str)
                ])

        return f"GimbalResult: [{struct_repr}]"

    @staticmethod
    def translate_from_rpc(rpcGimbalResult):
        """ Translates a gRPC struct to the SDK equivalent """
        return GimbalResult(
                
                GimbalResult.Result.translate_from_rpc(rpcGimbalResult.result),
                
                
                rpcGimbalResult.result_str
                )

    def translate_to_rpc(self, rpcGimbalResult):
        """ Translates this SDK object into its gRPC equivalent """

        
        
            
        self.result.translate_to_rpc(rpcGimbalResult.result)
            
        
        
        
            
        rpcGimbalResult.result_str = self.result_str
            
        
        



class GimbalError(Exception):
    """ Raised when a GimbalResult is a fail code """

    def __init__(self, result, origin, *params):
        self._result = result
        self._origin = origin
        self._params = params

    def __str__(self):
        return f"{self._result.result}: '{self._result.result_str}'; origin: {self._origin}; params: {self._params}"


class Gimbal(AsyncBase):
    """
     Provide control over a gimbal.

     Generated by dcsdkgen - MAVSDK Gimbal API
    """

    # Plugin name
    name = "Gimbal"

    def _setup_stub(self, channel):
        """ Setups the api stub """
        self._stub = gimbal_pb2_grpc.GimbalServiceStub(channel)

    
    def _extract_result(self, response):
        """ Returns the response status and description """
        return GimbalResult.translate_from_rpc(response.gimbal_result)
    

    async def set_pitch_and_yaw(self, pitch_deg, yaw_deg):
        """
         Set gimbal pitch and yaw angles.

         This sets the desired pitch and yaw angles of a gimbal.
         Will return when the command is accepted, however, it might
         take the gimbal longer to actually be set to the new angles.

         Parameters
         ----------
         pitch_deg : float
              Pitch angle in degrees (negative points down)

         yaw_deg : float
              Yaw angle in degrees (positive is clock-wise, range: -180 to 180 or 0 to 360)

         Raises
         ------
         GimbalError
             If the request fails. The error contains the reason for the failure.
        """

        request = gimbal_pb2.SetPitchAndYawRequest()
        request.pitch_deg = pitch_deg
        request.yaw_deg = yaw_deg
        response = await self._stub.SetPitchAndYaw(request)

        
        result = self._extract_result(response)

        if result.result is not GimbalResult.Result.SUCCESS:
            raise GimbalError(result, "set_pitch_and_yaw()", pitch_deg, yaw_deg)
        

    async def set_mode(self, gimbal_mode):
        """
         Set gimbal mode.

         This sets the desired yaw mode of a gimbal.
         Will return when the command is accepted. However, it might
         take the gimbal longer to actually be set to the new angles.

         Parameters
         ----------
         gimbal_mode : GimbalMode
              The mode to be set.

         Raises
         ------
         GimbalError
             If the request fails. The error contains the reason for the failure.
        """

        request = gimbal_pb2.SetModeRequest()
        
        gimbal_mode.translate_to_rpc(request.gimbal_mode)
                
            
        response = await self._stub.SetMode(request)

        
        result = self._extract_result(response)

        if result.result is not GimbalResult.Result.SUCCESS:
            raise GimbalError(result, "set_mode()", gimbal_mode)
        