# -*- coding: utf-8 -*-
from .._base import AsyncBase
from ..generated import param_pb2, param_pb2_grpc
from enum import Enum


class ParamResult:
    """
     Result type.

     Parameters
     ----------
     result : Result
          Result enum value

     result_str : std::string
          Human-readable English string describing the result

     """

    
    
    class Result(Enum):
        """
         Possible results returned for param requests.

         Values
         ------
         UNKNOWN
              Unknown error

         SUCCESS
              Request succeeded

         TIMEOUT
              Request timed out

         CONNECTION_ERROR
              Connection error

         WRONG_TYPE
              Wrong type

         PARAM_NAME_TOO_LONG
              Parameter name too long (> 16)

         """

        
        UNKNOWN = 0
        SUCCESS = 1
        TIMEOUT = 2
        CONNECTION_ERROR = 3
        WRONG_TYPE = 4
        PARAM_NAME_TOO_LONG = 5

        def translate_to_rpc(self, rpcResult):
            return {
                    0: param_pb2.ParamResult.UNKNOWN,
                    1: param_pb2.ParamResult.SUCCESS,
                    2: param_pb2.ParamResult.TIMEOUT,
                    3: param_pb2.ParamResult.CONNECTION_ERROR,
                    4: param_pb2.ParamResult.WRONG_TYPE,
                    5: param_pb2.ParamResult.PARAM_NAME_TOO_LONG
                }.get(self.value, None)

        @staticmethod
        def translate_from_rpc(rpc_enum_value):
            """ Parses a gRPC response """
            return {
                    0: ParamResult.Result.UNKNOWN,
                    1: ParamResult.Result.SUCCESS,
                    2: ParamResult.Result.TIMEOUT,
                    3: ParamResult.Result.CONNECTION_ERROR,
                    4: ParamResult.Result.WRONG_TYPE,
                    5: ParamResult.Result.PARAM_NAME_TOO_LONG,
                }.get(rpc_enum_value, None)

        def __str__(self):
            return self.name
    

    def __init__(
            self,
            result,
            result_str):
        """ Initializes the ParamResult object """
        self.result = result
        self.result_str = result_str

    def __equals__(self, to_compare):
        """ Checks if two ParamResult are the same """
        try:
            # Try to compare - this likely fails when it is compared to a non
            # ParamResult object
            return \
                (self.result == to_compare.result) and \
                (self.result_str == to_compare.result_str)

        except AttributeError:
            return False

    def __str__(self):
        """ ParamResult in string representation """
        struct_repr = ", ".join([
                "result: " + str(self.result),
                "result_str: " + str(self.result_str)
                ])

        return f"ParamResult: [{struct_repr}]"

    @staticmethod
    def translate_from_rpc(rpcParamResult):
        """ Translates a gRPC struct to the SDK equivalent """
        return ParamResult(
                
                ParamResult.Result.translate_from_rpc(rpcParamResult.result),
                
                
                rpcParamResult.result_str
                )

    def translate_to_rpc(self, rpcParamResult):
        """ Translates this SDK object into its gRPC equivalent """

        
        
            
        self.result.translate_to_rpc(rpcParamResult.result)
            
        
        
        
            
        rpcParamResult.result_str = self.result_str
            
        
        



class ParamError(Exception):
    """ Raised when a ParamResult is a fail code """

    def __init__(self, result, origin, *params):
        self._result = result
        self._origin = origin
        self._params = params

    def __str__(self):
        return f"{self._result.result}: '{self._result.result_str}'; origin: {self._origin}; params: {self._params}"


class Param(AsyncBase):
    """
     Provide raw access to get and set parameters.

     Generated by dcsdkgen - MAVSDK Param API
    """

    # Plugin name
    name = "Param"

    def _setup_stub(self, channel):
        """ Setups the api stub """
        self._stub = param_pb2_grpc.ParamServiceStub(channel)

    
    def _extract_result(self, response):
        """ Returns the response status and description """
        return ParamResult.translate_from_rpc(response.param_result)
    

    async def get_int_param(self, name):
        """
         Get an int parameter.

         If the type is wrong, the result will be `WRONG_TYPE`.

         Parameters
         ----------
         name : std::string
              Name of the parameter

         Returns
         -------
         value : int32_t
              Value of the requested parameter

         Raises
         ------
         ParamError
             If the request fails. The error contains the reason for the failure.
        """

        request = param_pb2.GetIntParamRequest()
        
            
        request.name = name
            
        response = await self._stub.GetIntParam(request)

        
        result = self._extract_result(response)

        if result.result is not ParamResult.Result.SUCCESS:
            raise ParamError(result, "get_int_param()", name)
        

        return response.value
        

    async def set_int_param(self, name, value):
        """
         Set an int parameter.

         If the type is wrong, the result will be `WRONG_TYPE`.

         Parameters
         ----------
         name : std::string
              Name of the parameter to set

         value : int32_t
              Value the parameter should be set to

         Raises
         ------
         ParamError
             If the request fails. The error contains the reason for the failure.
        """

        request = param_pb2.SetIntParamRequest()
        request.name = name
        request.value = value
        response = await self._stub.SetIntParam(request)

        
        result = self._extract_result(response)

        if result.result is not ParamResult.Result.SUCCESS:
            raise ParamError(result, "set_int_param()", name, value)
        

    async def get_float_param(self, name):
        """
         Get a float parameter.

         If the type is wrong, the result will be `WRONG_TYPE`.

         Parameters
         ----------
         name : std::string
              Name of the parameter

         Returns
         -------
         value : float
              Value of the requested parameter

         Raises
         ------
         ParamError
             If the request fails. The error contains the reason for the failure.
        """

        request = param_pb2.GetFloatParamRequest()
        
            
        request.name = name
            
        response = await self._stub.GetFloatParam(request)

        
        result = self._extract_result(response)

        if result.result is not ParamResult.Result.SUCCESS:
            raise ParamError(result, "get_float_param()", name)
        

        return response.value
        

    async def set_float_param(self, name, value):
        """
         Set a float parameter.

         If the type is wrong, the result will be `WRONG_TYPE`.

         Parameters
         ----------
         name : std::string
              Name of the parameter to set

         value : float
              Value the parameter should be set to

         Raises
         ------
         ParamError
             If the request fails. The error contains the reason for the failure.
        """

        request = param_pb2.SetFloatParamRequest()
        request.name = name
        request.value = value
        response = await self._stub.SetFloatParam(request)

        
        result = self._extract_result(response)

        if result.result is not ParamResult.Result.SUCCESS:
            raise ParamError(result, "set_float_param()", name, value)
        